@@ -26,8 +26,10 @@ pub struct Context {
 }
 
 pub fn current_context() -> Context {
-    let session =
-        env::var("ATUIN_SESSION").expect("failed to find ATUIN_SESSION - check your shell setup");
+    let Ok(session) = env::var("ATUIN_SESSION") else {
+        eprintln!("ERROR: Failed to find $ATUIN_SESSION in the environment. Check that you have correctly set up your shell.");
+        std::process::exit(1);
+    };
     let hostname = format!("{}:{}", whoami::hostname(), whoami::username());
     let cwd = match env::current_dir() {
         Ok(dir) => dir.display().to_string(),
@@ -446,7 +448,7 @@ impl Database for Sqlite {
                     } else if let Some(term) = query_part.strip_prefix('\'') {
                         format!("{glob}{term}{glob}")
                     } else if is_inverse {
-                        format!("{glob}{term}{glob}", term = query_part)
+                        format!("{glob}{query_part}{glob}")
                     } else {
                         query_part.split("").join(glob)
                     };
@@ -738,7 +740,7 @@ impl SqlBuilderExt for SqlBuilder {
         } else {
             cond.push_str(" LIKE '");
         }
-        cond.push_str(&esc(&mask.to_string()));
+        cond.push_str(&esc(mask.to_string()));
         cond.push('\'');
         if is_or {
             self.or_where(cond)
