@@ -4,10 +4,12 @@ use std::{
 };
 
 use chrono::{prelude::*, Utc};
+use clap::ValueEnum;
 use config::{Config, Environment, File as ConfigFile, FileFormat};
 use eyre::{eyre, Context, Result};
 use fs_err::{create_dir_all, File};
 use parse_duration::parse;
+use regex::RegexSet;
 use semver::Version;
 use serde::Deserialize;
 
@@ -16,7 +18,7 @@ pub const LAST_SYNC_FILENAME: &str = "last_sync_time";
 pub const LAST_VERSION_CHECK_FILENAME: &str = "last_version_check_time";
 pub const LATEST_VERSION_FILENAME: &str = "latest_version";
 
-#[derive(Clone, Debug, Deserialize, Copy)]
+#[derive(Clone, Debug, Deserialize, Copy, ValueEnum)]
 pub enum SearchMode {
     #[serde(rename = "prefix")]
     Prefix,
@@ -28,7 +30,7 @@ pub enum SearchMode {
     Fuzzy,
 }
 
-#[derive(Clone, Debug, Deserialize, Copy, PartialEq, Eq)]
+#[derive(Clone, Debug, Deserialize, Copy, PartialEq, Eq, ValueEnum)]
 pub enum FilterMode {
     #[serde(rename = "global")]
     Global = 0,
@@ -108,7 +110,12 @@ pub struct Settings {
     pub session_path: String,
     pub search_mode: SearchMode,
     pub filter_mode: FilterMode,
+    pub filter_mode_shell_up_key_binding: FilterMode,
+    pub shell_up_key_binding: bool,
     pub exit_mode: ExitMode,
+    #[serde(with = "serde_regex", default = "RegexSet::empty")]
+    pub history_filter: RegexSet,
+
     // This is automatically loaded when settings is created. Do not set in
     // config! Keep secrets and settings apart.
     pub session_token: String,
@@ -259,9 +266,8 @@ impl Settings {
         let data_dir = atuin_common::utils::data_dir();
 
         create_dir_all(&config_dir)
-            .wrap_err_with(|| format!("could not create dir {:?}", config_dir))?;
-        create_dir_all(&data_dir)
-            .wrap_err_with(|| format!("could not create dir {:?}", data_dir))?;
+            .wrap_err_with(|| format!("could not create dir {config_dir:?}"))?;
+        create_dir_all(&data_dir).wrap_err_with(|| format!("could not create dir {data_dir:?}"))?;
 
         let mut config_file = if let Ok(p) = std::env::var("ATUIN_CONFIG_DIR") {
             PathBuf::from(p)
@@ -288,6 +294,8 @@ impl Settings {
             .set_default("sync_address", "https://api.atuin.sh")?
             .set_default("search_mode", "fuzzy")?
             .set_default("filter_mode", "global")?
+            .set_default("filter_mode_shell_up_key_binding", "global")?
+            .set_default("shell_up_key_binding", false)?
             .set_default("exit_mode", "return-original")?
             .set_default("session_token", "")?
             .set_default("style", "auto")?
