@@ -3,18 +3,21 @@ use clap::Parser;
 use eyre::Result;
 
 use atuin_client::{
-    database::current_context, database::Database, history::History, settings::Settings,
+    database::current_context,
+    database::Database,
+    history::History,
+    settings::{FilterMode, SearchMode, Settings},
 };
 
 use super::history::ListMode;
 
 mod cursor;
 mod duration;
-mod event;
 mod history_list;
 mod interactive;
-pub use duration::format_duration;
+pub use duration::{format_duration, format_duration_into};
 
+#[allow(clippy::struct_excessive_bools)]
 #[derive(Parser)]
 pub struct Cmd {
     /// Filter search result by directory
@@ -49,6 +52,18 @@ pub struct Cmd {
     #[arg(long, short)]
     interactive: bool,
 
+    /// Allow overriding filter mode over config
+    #[arg(long = "filter-mode")]
+    filter_mode: Option<FilterMode>,
+
+    /// Allow overriding search mode over config
+    #[arg(long = "search-mode")]
+    search_mode: Option<SearchMode>,
+
+    /// Marker argument used to inform atuin that it was invoked from a shell up-key binding (hidden from help to avoid confusion)
+    #[arg(long = "shell-up-key-binding", hide = true)]
+    shell_up_key_binding: bool,
+
     /// Use human-readable formatting for time
     #[arg(long)]
     human: bool,
@@ -58,13 +73,25 @@ pub struct Cmd {
     /// Show only the text of the command
     #[arg(long)]
     cmd_only: bool,
+
+    /// Available variables: {command}, {directory}, {duration}, {user}, {host} and {time}.
+    /// Example: --format "{time} - [{duration}] - {directory}$\t{command}"
+    #[arg(long, short)]
+    format: Option<String>,
 }
 
 impl Cmd {
-    pub async fn run(self, db: &mut impl Database, settings: &Settings) -> Result<()> {
+    pub async fn run(self, db: &mut impl Database, settings: &mut Settings) -> Result<()> {
+        if self.search_mode.is_some() {
+            settings.search_mode = self.search_mode.unwrap();
+        }
+        if self.filter_mode.is_some() {
+            settings.filter_mode = self.filter_mode.unwrap();
+        }
+        settings.shell_up_key_binding = self.shell_up_key_binding;
         if self.interactive {
             let item = interactive::history(&self.query, settings, db).await?;
-            eprintln!("{}", item);
+            eprintln!("{item}");
         } else {
             let list_mode = ListMode::from_flags(self.human, self.cmd_only);
             let entries = run_non_interactive(
@@ -74,6 +101,7 @@ impl Cmd {
                 self.exit,
                 self.exclude_exit,
                 self.exclude_cwd,
+                self.format,
                 self.before,
                 self.after,
                 self.limit,
@@ -99,6 +127,7 @@ async fn run_non_interactive(
     exit: Option<i64>,
     exclude_exit: Option<i64>,
     exclude_cwd: Option<String>,
+    format: Option<String>,
     before: Option<String>,
     after: Option<String>,
     limit: Option<i64>,
@@ -179,6 +208,6 @@ async fn run_non_interactive(
         .map(std::borrow::ToOwned::to_owned)
         .collect();
 
-    super::history::print_list(&results, list_mode);
+    super::history::print_list(&results, list_mode, format.as_deref());
     Ok(results.len())
 }
